
from string import  find, rfind
    
def import_trianglulation(ofile):
    """
    import a file, ofile, with the format
    
    First line:  <# of vertices> <# of attributes>
    Following lines:  <vertex #> <x> <y> [attributes]
    One line:  <# of triangles> 
    Following lines:  <triangle #> <vertex #>  <vertex #> <vertex #> <neigbouring triangle #> <neigbouring triangle #> <neigbouring triangle #> [attribute of region]
    One line:  <# of segments> 
    Following lines:  <segment #> <vertex #>  <vertex #> [boundary marker]
    Note: This might throw a can't load file error
    """
    fd = open(ofile,'r')
    dict = read_trianglulation(fd)
    dict_mesh = read_mesh(fd)
    for element in dict_mesh.keys():
        dict[element] = dict_mesh[element]
        
    fd.close()
    return dict


def read_trianglulation(fd):
    delimiter = " "
    ######### loading the point info
    line = fd.readline()
    #print line
    fragments = line.split()
    #for fragment in fragments:
    #    print fragment
    NumOfVertices = fragments[0]
    NumOfVertAttributes = fragments[1]
    points = []
    pointattributes = []
    for index in range(int(NumOfVertices)):        
        #print index
        fragments = fd.readline().split()
        #print fragments
        fragments.pop(0) #pop off the index
        
        # pop the x y off so we're left with a list of attributes       
        vert = [float(fragments.pop(0)),float(fragments.pop(0))]
        points.append(vert)
        apointattributes  = []
        #print fragments
        for fragment in fragments:
            apointattributes.append(float(fragment))
        pointattributes.append(apointattributes)
        
    ######### loading the triangle info
    line = fd.readline()
    #print line
    fragments = line.split()
    #for fragment in fragments:
    #    print fragment
    NumOfTriangles = fragments[0]
    triangles = []
    triangleattributes = []
    triangleneighbors = []
    for index in range(int(NumOfTriangles)):        
        #print index
        line = fd.readline()
        line.strip() # so we can get the region string
        fragments = line.split()
        #print fragments
        fragments.pop(0) #pop off the index
        
        tri = [int(fragments[0]),int(fragments[1]),int(fragments[2])]
        triangles.append(tri)
        neighbors = [int(fragments[3]),int(fragments[4]),int(fragments[5])]
        triangleneighbors.append(neighbors)
        for x in range(7): # remove index [<vertex #>] [<neigbouring tri #>]
            line = line[find(line,delimiter):] # remove index
            line = line.lstrip()
        stringmarker = line.strip()
        triangleattributes.append([stringmarker])
        
    ######### loading the segment info
    line = fd.readline()
    #print line
    fragments = line.split()
    #for fragment in fragments:
    #    print fragment
    NumOfSegments = fragments[0]
    segments = []
    segmentmarkers = []
    for index in range(int(NumOfSegments)):        
        #print index
        line = fd.readline()
        line.strip() # to get the segment string
        fragments = line.split()
        #print fragments
        fragments.pop(0) #pop off the index
        seg = [int(fragments[0]),int(fragments[1])]
        segments.append(seg)
        line = line[find(line,delimiter):] # remove index
        line = line.lstrip()
        line = line[find(line,delimiter):] # remove x
        line = line.lstrip()
        line = line[find(line,delimiter):] # remove y
        stringmarker = line.strip()
        segmentmarkers.append(stringmarker)
    meshDict = {}
    meshDict['generatedpointlist'] = points
    meshDict['generatedpointattributelist'] = pointattributes
    meshDict['generatedtrianglelist'] = triangles
    meshDict['generatedtriangleattributelist'] = triangleattributes
    meshDict['generatedtriangleneighborlist'] = triangleneighbors 
    meshDict['generatedsegmentlist'] = segments 
    meshDict['generatedsegmentmarkerlist'] = segmentmarkers
    return meshDict
    
def import_mesh(ofile):
    """
    import a file, ofile, with the format
    
    First line:  <# of vertices> <# of attributes>
    Following lines: <x> <y> [attributes]
    One line:  <# of segments> 
    Following lines:  <vertex #>  <vertex #> [boundary marker]
    Note: This might throw a can't load file error
    """
    fd = open(ofile,'r')
    meshDict = read_mesh(fd)
    fd.close()
    return meshDict

def read_mesh(fd):
    delimiter = " " # warning: split() calls are using default whitespace
    
    ######### loading the point info
    line = fd.readline()
    #print line
    fragments = line.split()
    #for fragment in fragments:
    #    print fragment
    NumOfVertices = fragments[0]
    NumOfVertAttributes = fragments[1]
    points = []
    pointattributes = []
    for index in range(int(NumOfVertices)):        
        #print index
        fragments = fd.readline().split() 
        #print fragments
        fragments.pop(0) #pop off the index
        # pop the x y off so we're left with a list of attributes
        vert = [float(fragments.pop(0)),float(fragments.pop(0))]
        points.append(vert)
        apointattributes  = []
        #print fragments
        for fragment in fragments:
            apointattributes.append(float(fragment))
        pointattributes.append(apointattributes)
        

    ######### loading the segment info
    line = fd.readline()
    #print line
    fragments = line.split()
    #for fragment in fragments:
    #    print fragment
    NumOfSegments = fragments[0]
    segments = []
    segmentmarkers = []
    for index in range(int(NumOfSegments)):  
        #print index
        line = fd.readline()
        fragments = line.split()
        #print fragments
        fragments.pop(0) #pop off the index
        
        seg = [int(fragments[0]),int(fragments[1])]
        segments.append(seg)
        line = line[find(line,delimiter):] # remove index
        line = line.lstrip()
        line = line[find(line,delimiter):] # remove x
        line = line.lstrip()
        line = line[find(line,delimiter):] # remove y
        stringmarker = line.strip()
        segmentmarkers.append(stringmarker)  

    ######### loading the hole info
    line = fd.readline()
    #print line
    fragments = line.split()
    #for fragment in fragments:
    #    print fragment
    numOfHoles = fragments[0]
    holes = []
    for index in range(int(numOfHoles)):        
        #print index
        fragments = fd.readline().split()
        #print fragments
        fragments.pop(0) #pop off the index
        hole = [float(fragments[0]),float(fragments[1])]
        holes.append(hole)

    
    ######### loading the region info
    line = fd.readline()
    #print line
    fragments = line.split()
    #for fragment in fragments:
    #    print fragment
    numOfRegions = fragments[0]
    regions = []
    regionattributes = []
    for index in range(int(numOfRegions)):
        line = fd.readline()
        fragments = line.split()
        #print fragments
        fragments.pop(0) #pop off the index
        region = [float(fragments[0]),float(fragments[1])]
        regions.append(region)

        line = line[find(line,delimiter):] # remove index
        line = line.lstrip()
        line = line[find(line,delimiter):] # remove x
        line = line.lstrip()
        line = line[find(line,delimiter):] # remove y
        stringmarker = line.strip()
        regionattributes.append(stringmarker)
    regionmaxareas = []
    for index in range(int(numOfRegions)): # Read in the Max area info
        line = fd.readline()
        fragments = line.split()
        #print fragments
        fragments.pop(0) #pop off the index
        if len(fragments) == 0: #no max area
            regionmaxareas.append(None)
        else:
            regionmaxareas.append(float(fragments[0]))

        
    meshDict = {}
    meshDict['pointlist'] = points
    meshDict['pointattributelist'] = pointattributes
    meshDict['segmentlist'] = segments 
    meshDict['segmentmarkerlist'] = segmentmarkers
    meshDict['holelist'] = holes
    meshDict['regionlist'] = regions
    meshDict['regionattributelist'] = regionattributes
    meshDict['regionmaxarealist'] = regionmaxareas

    return meshDict

if __name__ == "__main__":
    m = import_mesh("tee.txt")
    print m
